﻿--[[

	Quest Completist - qcCore.lua
	Written by: Alistair Maxwell

--]]

--[[ Tables ]]--
qcZoneQuests = {}
qcZoneQuestsV2 = {}
qcQuestHistory = {}
qcOptions = {}
qcUnknownQuests = {}
qcNPC = {}
qcQuest = {}
qcParsedData = {}
qcDynamicDB = {}
qcStaticDB = {}
qcDBChanges = {}
qcCleanZones = {}
qcConvertedDB = {}
qcCompletedQuests = {}

local L = QuestCompletistLocalization

--[[ Declare vars ]]--
local qcCurrentZone = 0
local qcCurrentZoneQuestCount = 0
local qcCurrentScrollPosition = 1
local qcOnUpdateInterval = 1
local qcOnUpdateElapsed = 0
local qcTimeToDailyReset = 0
qcPlayerEnglishClass = ""
qcPlayerLocalizedClass = ""
qcPlayerEnglishFaction = ""
qcPlayerLocalizedFaction = ""
qcPlayerEnglishRace = ""
qcPlayerLocalizedRace = ""

local QCADDON_MSG_PREFIX = "QCOMPLETIST"
local QCADDON_VERSION = 86
local QCADDON_DELIMITER = ":"
local QCSINGLEPADDING = " "
local QCDOUBLEPADDING = "  "
local QCTRIPLEPADDING = "   "
QCADDON_CHAT_TITLE = "|cFF9482C9Quest Completist:|r "
QCADDON_CONTACT_EMAIL = "viduus.wow@gmail.com"
QCADDON_SV_LOCATION = "\\WTF\\Account\\<ACCOUNT NAME>\\SavedVariables\\QuestCompletist.lua"

--	Constants for the Key Bindings & Slash Commands --
BINDING_HEADER_QCQUESTCOMPLETIST = "Quest Completist";
BINDING_NAME_QCTOGGLEFRAME = "Toggle Frame";
SLASH_QUESTCOMPLETIST1 = "/qc"
SLASH_QUESTCOMPLETIST2 = "/questc"

SlashCmdList["QUESTCOMPLETIST"] = function(msg, editbox)
	ShowUIPanel(frameQuestCompletist)
end

function qcScrollUpdate(value)

	--[[
	Do this to ensure we are not pointlessly doing extra work when the
	slider value hasn't changed.
	--]]

	if (qcCurrentScrollPosition ~= value) then
		qcCurrentScrollPosition = value
		qcUpdateMenu(nil, value)
	end

end

function qcUpdateCurrentZoneText(qcZoneID)

	-- TODO: Possible update to use quest categories not continent zones?

	for qcIndex, qcEntry in pairs(qcContinentZones) do
		if (qcEntry[2] == qcZoneID) then
			frameQuestCompletist.selectedZone:SetText(qcEntry[3])
			break
		else
			frameQuestCompletist.selectedZone:SetText(QCSINGLEPADDING)
		end
	end

end

function qcUpdateMenu(qcZoneID, qcStartItem, qcDailyReset)

	local qcOffset
	local qcMenuButton = "qcMenuButton"
	local qcOpenLevel = "["
	local qcCloseLevel = "] "

	if (qcZoneID ~= nil) then --[[ We need to collect data for a different zone ]]--
		qcCurrentZone = qcZoneID --[[ Store the current zone ID for use by other functions ]]--
		qcUpdateCurrentZoneText(qcZoneID)
		--qcGetZoneQuests(qcZoneID)
		qcGetZoneQuestsTest(qcZoneID)
		qcCurrentZoneQuestCount = (#qcZoneQuestsV2) --[[ Store the current zone quest count for use by other functions ]]--
		frameQuestCompletist.qcCurrentZoneQuestCount:SetText(qcCurrentZoneQuestCount .. " Quests Found")
		if (qcCurrentZoneQuestCount <= 0) then --[[ No quests were found for this zone ]]--
			qcMenuSlider:SetMinMaxValues(1, 1)
		elseif (qcCurrentZoneQuestCount < 16) then --[[ All quests can fit on one screen ]]--
			qcMenuSlider:SetMinMaxValues(1, 1)
		else --[[ Not all quests can fit on one screen ]]--
			qcMenuSlider:SetMinMaxValues(1, qcCurrentZoneQuestCount - 15)
		end
		qcMenuSlider:SetValue(qcStartItem)
	end

	for qcMenuIndex = 1, 16 do --[[ 16 quests can be shown at any one time ]]--
		qcOffset = ((qcMenuIndex + qcStartItem) - 1)
		if (qcCurrentZoneQuestCount >= qcOffset) then --[[ We only want quests qhich are after the start item ]]--
			_G[qcMenuButton .. qcMenuIndex].QuestName:SetText(qcOpenLevel .. qcZoneQuestsV2[qcOffset][4] .. qcCloseLevel .. qcZoneQuestsV2[qcOffset][5])
			_G[qcMenuButton .. qcMenuIndex].QuestTag:SetText(qcZoneQuestsV2[qcOffset][2])
			if (qcZoneQuestsV2[qcOffset][11] ~= nil) then
				_G[qcMenuButton .. qcMenuIndex].QuestProviderID:SetText(qcZoneQuestsV2[qcOffset][11][1])
				_G[qcMenuButton .. qcMenuIndex].QuestProviderName:SetText(qcZoneQuestsV2[qcOffset][11][2])
				_G[qcMenuButton .. qcMenuIndex].QuestProviderZone:SetText(qcZoneQuestsV2[qcOffset][11][3])
				_G[qcMenuButton .. qcMenuIndex].QuestProviderX:SetText(qcZoneQuestsV2[qcOffset][11][4])
				_G[qcMenuButton .. qcMenuIndex].QuestProviderY:SetText(qcZoneQuestsV2[qcOffset][11][5])
				_G[qcMenuButton .. qcMenuIndex].QuestProviderComments:SetText(qcZoneQuestsV2[qcOffset][11][6])
			else
				_G[qcMenuButton .. qcMenuIndex].QuestProviderID:SetText(nil)
				_G[qcMenuButton .. qcMenuIndex].QuestProviderName:SetText(nil)
				_G[qcMenuButton .. qcMenuIndex].QuestProviderZone:SetText(nil)
				_G[qcMenuButton .. qcMenuIndex].QuestProviderX:SetText(nil)
				_G[qcMenuButton .. qcMenuIndex].QuestProviderY:SetText(nil)
				_G[qcMenuButton .. qcMenuIndex].QuestProviderComments:SetText(nil)
			end
			if (qcZoneQuestsV2[qcOffset][3] == 1) then --[[ Quest type is normal ]]--
				_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Show()
				_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(1.0, 1.0, 1.0, 1.0)
			elseif (qcZoneQuestsV2[qcOffset][3] == 2) then --[[ Quest type is repeatable ]]--
				_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Show()
				_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(0.0941176470588235, 0.6274509803921569, 0.9411764705882353, 1.0)
			elseif (qcZoneQuestsV2[qcOffset][3] == 3) then --[[ Quest type is daily ]]--
				_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].DailyIcon:Show()
				_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(0.0941176470588235, 0.6274509803921569, 0.9411764705882353, 1.0)
			elseif (qcZoneQuestsV2[qcOffset][3] == 4) then --[[ Quest type is special ]]--
				_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Show()
				_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(1.0, 0.6156862745098039, 0.0862745098039216, 1.0)
			else --[[ Quest type unknown ]]--
				_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(1.0, 1.0, 1.0, 1.0)
			end
			if (qcZoneQuestsV2[qcOffset][6] == "A") then --[[ Alliance only ]]--
				_G[qcMenuButton .. qcMenuIndex].HordeIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].AllianceIcon:Show()
			elseif (qcZoneQuestsV2[qcOffset][6] == "H") then --[[ Horde only ]]--
				_G[qcMenuButton .. qcMenuIndex].AllianceIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].HordeIcon:Show()
			elseif (qcZoneQuestsV2[qcOffset][6] == "AH") or (qcZoneQuestsV2[qcOffset][6] == "HA") then --[[ Both sides ]]--
				_G[qcMenuButton .. qcMenuIndex].AllianceIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].HordeIcon:Hide()
			elseif (qcZoneQuestsV2[qcOffset][6] == 4) then --[[ No side data ]]--
				_G[qcMenuButton .. qcMenuIndex].AllianceIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].HordeIcon:Hide()
			else --[[ Unknown side ]]--
				_G[qcMenuButton .. qcMenuIndex].AllianceIcon:Hide()
				_G[qcMenuButton .. qcMenuIndex].HordeIcon:Hide()
			end
			--[[ Now check if the quest is in progress, or completed but not handed in ]]--
			if (GetQuestLogIndexByID(qcZoneQuestsV2[qcOffset][2]) ~= 0) then -- This quest is currently in our quest log --
				local _S
				local qcIsComplete
				_S, _S, _S, _S, _S, _S, qcIsComplete, _S, _S = GetQuestLogTitle(GetQuestLogIndexByID(qcZoneQuestsV2[qcOffset][2]))
				if (qcIsComplete == nil) then --[[ Quest is yet to reach a conclusion ]]--
					_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Show()
					_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(0.5803921568627451, 0.5882352941176471, 0.5803921568627451, 1.0)
				elseif (qcIsComplete == 1) then --[[ Is completed ]]--
					_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Show()
					_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(1.0, 0.8196078431372549, 0.0, 1.0)
				elseif (qcIsComplete == -1) then --[[ Is failed or abandoned ]]--
					_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
					_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Show()
					_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(0.9372549019607843, 0.1490196078431373, 0.0627450980392157, 1.0)
				end
			end
			--[[
			Now that the basic colours for the quest types have been applied,
			go through the completed quests DB for the character to see what
			has been completed, and colour as needed. Does not colour repeatables
			or daily quests.
			--]]
			if (qcCompletedQuests[qcZoneQuestsV2[qcOffset][2]] == nil) then
				--[[ Quest isn't flagged as being completed ]]--
			else
				if (qcZoneQuestsV2[qcOffset][3] ~= 2) and (qcZoneQuestsV2[qcOffset][3] ~= 3) then --[[ Quest isn't repeatable or daily ]]--
					if (qcCompletedQuests[qcZoneQuestsV2[qcOffset][2]]["I"] == 1) then --[[ User wants to ignore this quest ]]--
						_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(0.58, 0.51, 0.79, 1.0)
						_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Show()
					else
						if (qcCompletedQuests[qcZoneQuestsV2[qcOffset][2]]["C"] == 1) then --[[ Quest is complete ]]--
							_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(0.0, 1.0, 0.0, 1.0)
							_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Show()
						else
							_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(1.0, 1.0, 1.0, 1.0)
							_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
							_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Show()
						end
					end
				else
					if (qcZoneQuestsV2[qcOffset][3] == 3) then --[[ Daily quest ]]--
						_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(0.0941176470588235, 0.6274509803921569, 0.9411764705882353, 1.0)
						_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].DailyIcon:Show()
					elseif (qcZoneQuestsV2[qcOffset][3] == 2) then --[[ Repeatable quest ]]--
						_G[qcMenuButton .. qcMenuIndex].QuestName:SetTextColor(0.0941176470588235, 0.6274509803921569, 0.9411764705882353, 1.0)
						_G[qcMenuButton .. qcMenuIndex].AvailableIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].DailyIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].SpecialIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].IncompleteIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].ActiveIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].CompleteIcon:Hide()
						_G[qcMenuButton .. qcMenuIndex].RepeatableIcon:Show()
					end
				end
			end
			_G[qcMenuButton .. qcMenuIndex]:Show()
			_G[qcMenuButton .. qcMenuIndex]:Enable()
		else
			_G[qcMenuButton .. qcMenuIndex].QuestName:SetText(QCSINGLEPADDING)
			_G[qcMenuButton .. qcMenuIndex]:Hide()
			_G[qcMenuButton .. qcMenuIndex]:Disable()
		end
	end

end

function qcQuestQueryCompleted()

	--TODO: I dont use qcUnknownQuestCount, strings need updated

	local qcRecievedQuestCount = 0
	local qcAddedToQuestHistory = 0
	local qcUnknownQuestCount = 0
	local qcCompletedTable = {}

	wipe(qcCompletedTable)
	wipe(qcQuestHistory) --TODO: Table no longer used
	--wipe(qcCompletedQuests) Cant wipe or it will clear manually set quests
	GetQuestsCompleted(qcCompletedTable)

	for qcCompletedTableIndex, qcCompletedQuest in pairs(qcCompletedTable) do
		qcRecievedQuestCount = (qcRecievedQuestCount + 1)
		if (qcCompletedQuests[qcCompletedTableIndex] == nil) then
			qcAddedToQuestHistory = (qcAddedToQuestHistory + 1)
			qcCompletedQuests[qcCompletedTableIndex] = {}
		end
		qcCompletedQuests[qcCompletedTableIndex]["C"] = 1 --[[ Completed ]]--
	end

	print(QCADDON_CHAT_TITLE .. string.format(L["QC_QUERYRESULTS"], qcRecievedQuestCount, qcAddedToQuestHistory, qcUnknownQuestCount))
	qcUpdateMenu(nil, qcMenuSlider:GetValue())

end

function qcGetZoneQuestsTest(zoneID)

	wipe(qcZoneQuestsV2)

	--[[ Pull all quests related to this zone from the database ]]--
	for qcQuestDatabaseIndex, qcQuest in pairs(qcQuestDatabases) do
		if (qcQuest[1] == zoneID) then --[[ This quest is from the required zone ]]--
			table.insert(qcZoneQuestsV2, qcQuest)
		end
	end

	--[[ Completed filtering ]]--
	if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then
		for qcZoneQuestsIndex = #qcZoneQuestsV2, 1, -1 do
			if (qcCompletedQuests[qcZoneQuestsV2[qcZoneQuestsIndex][2]] ~= nil) then
				if (qcCompletedQuests[qcZoneQuestsV2[qcZoneQuestsIndex][2]]["C"] == 1) then
					table.remove(qcZoneQuestsV2, qcZoneQuestsIndex)
				end
			end
		end
	end

	--[[ Faction filtering ]]--
	if (_G["qcFactionCheckButton"]:GetChecked() == 1) then
		if (string.upper(qcPlayerEnglishFaction) == "ALLIANCE") then
			for qcZoneQuestsIndex = #qcZoneQuestsV2, 1, -1 do
				if (string.find(qcZoneQuestsV2[qcZoneQuestsIndex][6], "A", 1, true) == nil) then
					table.remove(qcZoneQuestsV2, qcZoneQuestsIndex)
				end
			end
		elseif (string.upper(qcPlayerEnglishFaction) == "HORDE") then
			for qcZoneQuestsIndex = #qcZoneQuestsV2, 1, -1 do
				if (string.find(qcZoneQuestsV2[qcZoneQuestsIndex][6], "H", 1, true) == nil) then
					table.remove(qcZoneQuestsV2, qcZoneQuestsIndex)
				end
			end
		end
	end

	--[[ Class filtering ]]--
	--if (_G["qcClassCheckButton"]:GetChecked() == 1) then
	--	for qcZoneQuestsIndex = #qcZoneQuestsV2, 1, -1 do
	--		if (string.find(qcZoneQuestsV2[qcZoneQuestsIndex][8], string.upper(qcPlayerEnglishClass), 1, true) == nil) then
	--			table.remove(qcZoneQuestsV2, qcZoneQuestsIndex)
	--		end
	--	end
	--end

	--[[ Race filtering ]]--
	--if (_G["qcClassCheckButton"]:GetChecked() == 1) then
	--	for qcZoneQuestsIndex = #qcZoneQuestsV2, 1, -1 do
	--		if (string.find(qcZoneQuestsV2[qcZoneQuestsIndex][10], string.upper(qcPlayerEnglishRace), 1, true) == nil) then
	--			table.remove(qcZoneQuestsV2, qcZoneQuestsIndex)
	--		end
	--	end
	--end

	--[[ Sort the zones quests ]]--
	if (qcSettings["SORT"] == 1) then
		table.sort(qcZoneQuestsV2, function(a, b) return (a[4]<b[4] or (a[4] == b[4] and a[5]<b[5])) end)
	elseif (qcSettings["SORT"] == 2) then
		table.sort(qcZoneQuestsV2, function(a, b) return a[5]<b[5] end)
	else --[[ No sort preference found, so sorting by level ]]--
		table.sort(qcZoneQuestsV2, function(a, b) return (a[4]<b[4] or (a[4] == b[4] and a[5]<b[5])) end)
	end

end

function qcGetZoneQuests(zoneID)

	wipe(qcZoneQuests)

	for qcQuestDatabaseIndex, qcQuestDatabase in pairs(qcQuestDatabases) do
		if (qcQuestDatabase[1] == zoneID) then -- This is a quest from the required zone --
			if (_G["qcFactionCheckButton"]:GetChecked() == 1) then -- We only want players faction quests (Including quests set as Both, etc) --
				if (string.upper(qcPlayerEnglishFaction) == "ALLIANCE") then
					if (string.find(qcQuestDatabase[6], "A", 1, true) ~= nil) then
						if (_G["qcClassCheckButton"]:GetChecked() == 1) then -- We dont want to includes quests that are for other classes --
							if (qcQuestDatabase[7] == 1) then -- This is a class specific quest --
								if (string.find(qcQuestDatabase[8], string.upper(qcPlayerEnglishClass), 1, true) ~= nil) then -- This quest is for the players class --
									if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then -- We dont want to see completed quests --
										if (qcCompletedQuests[qcQuestDatabase[2]]["C"] == 0) or (qcCompletedQuests[qcQuestDatabase[2]]["C"] == nil) then -- Quest has not been completed in the past --
											table.insert(qcZoneQuests, qcQuestDatabase)
										end
									else
										table.insert(qcZoneQuests, qcQuestDatabase)
									end
								end
							else
								if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then -- We dont want to see completed quests --
									if (qcCompletedQuests[qcQuestDatabase[2]]["C"] == 0) or (qcCompletedQuests[qcQuestDatabase[2]]["C"] == nil) then -- Quest has not been completed in the past --
										table.insert(qcZoneQuests, qcQuestDatabase)
									end
								else
									table.insert(qcZoneQuests, qcQuestDatabase)
								end
							end
						else
							if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then -- We dont want to see completed quests --
								if (qcCompletedQuests[qcQuestDatabase[2]]["C"] == 0) or (qcCompletedQuests[qcQuestDatabase[2]]["C"] == nil) then -- Quest has not been completed in the past --
									table.insert(qcZoneQuests, qcQuestDatabase)
								end
							else
								table.insert(qcZoneQuests, qcQuestDatabase)
							end
						end
					end
				elseif (string.upper(qcPlayerEnglishFaction) == "HORDE") then
					if (string.find(qcQuestDatabase[6], "H", 1, true) ~= nil) then
						if (_G["qcClassCheckButton"]:GetChecked() == 1) then -- We dont want to includes quests that are for other classes --
							if (qcQuestDatabase[7] == 1) then -- This is a class specific quest --
								if (string.find(qcQuestDatabase[8], string.upper(qcPlayerEnglishClass), 1, true) ~= nil) then -- This quest is for the players class --
									if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then -- We dont want to see completed quests --
										if (qcCompletedQuests[qcQuestDatabase[2]]["C"] == 0) or (qcCompletedQuests[qcQuestDatabase[2]]["C"] == nil) then -- Quest has not been completed in the past --
											table.insert(qcZoneQuests, qcQuestDatabase)
										end
									else
										table.insert(qcZoneQuests, qcQuestDatabase)
									end
								end
							else
								if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then -- We dont want to see completed quests --
									if (qcCompletedQuests[qcQuestDatabase[2]]["C"] == 0) or (qcCompletedQuests[qcQuestDatabase[2]]["C"] == nil) then -- Quest has not been completed in the past --
										table.insert(qcZoneQuests, qcQuestDatabase)
									end
								else
									table.insert(qcZoneQuests, qcQuestDatabase)
								end
							end
						else
							if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then -- We dont want to see completed quests --
								if (qcCompletedQuests[qcQuestDatabase[2]]["C"] == 0) or (qcCompletedQuests[qcQuestDatabase[2]]["C"] == nil) then -- Quest has not been completed in the past --
									table.insert(qcZoneQuests, qcQuestDatabase)
								end
							else
								table.insert(qcZoneQuests, qcQuestDatabase)
							end
						end
					end
				end
			else -- Get all factions quests --
				if (_G["qcClassCheckButton"]:GetChecked() == 1) then -- We dont want to includes quests that are for other classes --
					if (qcQuestDatabase[7] == 1) then -- This is a class specific quest --
						if (string.find(qcQuestDatabase[8], string.upper(qcPlayerEnglishClass), 1, true) ~= nil) then -- This quest is for the players class --
							if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then -- We dont want to see completed quests --
								if (qcCompletedQuests[qcQuestDatabase[2]]["C"] == 0) or (qcCompletedQuests[qcQuestDatabase[2]]["C"] == nil) then -- Quest has not been completed in the past --
									table.insert(qcZoneQuests, qcQuestDatabase)
								end
							else
								table.insert(qcZoneQuests, qcQuestDatabase)
							end
						end
					else
						if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then -- We dont want to see completed quests --
							if (qcCompletedQuests[qcQuestDatabase[2]]["C"] == 0) or (qcCompletedQuests[qcQuestDatabase[2]]["C"] == nil) then -- Quest has not been completed in the past --
								table.insert(qcZoneQuests, qcQuestDatabase)
							end
						else
							table.insert(qcZoneQuests, qcQuestDatabase)
						end
					end
				else
					if (_G["qcCompletedCheckButton"]:GetChecked() == 1) then -- We dont want to see completed quests --
						if (qcCompletedQuests[qcQuestDatabase[2]]["C"] == 0) or (qcCompletedQuests[qcQuestDatabase[2]]["C"] == nil) then -- Quest has not been completed in the past --
							table.insert(qcZoneQuests, qcQuestDatabase)
						end
					else
						table.insert(qcZoneQuests, qcQuestDatabase)
					end
				end
			end
		end
	end

	--[[ Sort the zones quests ]]--
	if (qcSettings["SORT"] == 1) then
		table.sort(qcZoneQuests, function(a, b) return (a[4]<b[4] or (a[4] == b[4] and a[5]<b[5])) end)
	elseif (qcSettings["SORT"] == 2) then
		table.sort(qcZoneQuests, function(a, b) return a[5]<b[5] end)
	else --[[ No sort preference found, so sorting by level ]]--
		table.sort(qcZoneQuests, function(a, b) return (a[4]<b[4] or (a[4] == b[4] and a[5]<b[5])) end)
	end

end

local function qcClearUpdateCache()

	--[[ Clear the caches ]]--
	wipe(qcNPC)
	wipe(qcLoggerData)
	wipe(qcDynamicDB)

end

function qcMenuMouseWheel(self, delta)

	local currentPosition = qcMenuSlider:GetValue()

	if (delta < 0) and (currentPosition < qcCurrentZoneQuestCount) then
		qcMenuSlider:SetValue(currentPosition + 2)
	elseif (delta > 0) and (currentPosition > 1) then
		qcMenuSlider:SetValue(currentPosition - 2)
	end

end

function qcZoneDropdown_Initialize(self, level)

local qcDropDownInfo

	if (level == 1) then

		--[[ Continents & Continent Zones ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Continents"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		for qcContinentIndex, qcContinent in ipairs(qcContinents) do
			qcDropDownInfo = UIDropDownMenu_CreateInfo()
			qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. qcContinent[2])
			qcDropDownInfo.notCheckable = true
			qcDropDownInfo.hasArrow = true
			qcDropDownInfo.value = qcContinent[1]
			UIDropDownMenu_AddButton(qcDropDownInfo, level)
		end
		--[[ Dungeon\Raid Categories and Dungeons\Raids ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Dungeons & Raids"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		for qcDungeonCategoryIndex, qcDungeonCategory in ipairs(qcDungeonCategories) do
			qcDropDownInfo = UIDropDownMenu_CreateInfo()
			qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. qcDungeonCategory[2])
			qcDropDownInfo.notCheckable = true
			qcDropDownInfo.hasArrow = true
			qcDropDownInfo.value = qcDungeonCategory[1]
			UIDropDownMenu_AddButton(qcDropDownInfo, level)
		end
		--[[ Class Quests ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Class Quests"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Classes")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.hasArrow = true
		qcDropDownInfo.value = 20
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		--[[ Miscellaneous Section ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Miscellaneous"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)
		for qcMiscellaneousCategoriesIndex, qcMiscellaneousCategory in ipairs(qcMiscellaneousCategories) do
			qcDropDownInfo = UIDropDownMenu_CreateInfo()
			qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. qcMiscellaneousCategory[2])
			qcDropDownInfo.notCheckable = true
			qcDropDownInfo.hasArrow = true
			qcDropDownInfo.value = qcMiscellaneousCategory[1]
			UIDropDownMenu_AddButton(qcDropDownInfo, level)
		end
		--[[ Options ]]--
		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = "Options"
		qcDropDownInfo.isTitle = 1
		qcDropDownInfo.notCheckable = true
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Perform Server Query...")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.colorCode = COLOUR_DRUID
		qcDropDownInfo.arg1 = "SERVERQUERY"
		function qcDropDownInfo.func(button, arg1)
			print(QCADDON_CHAT_TITLE .. L["QC_QUERYREQUESTED"])
			frameQuestCompletist:RegisterEvent("QUEST_QUERY_COMPLETE")
			QueryQuestsCompleted()
		end
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Clear Update Cache...")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.colorCode = COLOUR_DRUID
		qcDropDownInfo.arg1 = "CLEARUPDATECACHE"
		function qcDropDownInfo.func(button, arg1)
			print(QCADDON_CHAT_TITLE .. "Clearing your update Cache...")
			qcClearUpdateCache()
		end
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Sort: Level")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.colorCode = COLOUR_DRUID
		qcDropDownInfo.arg1 = "SORTLEVEL"
		function qcDropDownInfo.func(button, arg1)
			qcSettings["SORT"] = 1
			qcUpdateMenu(qcCurrentZone, qcMenuSlider:GetValue())
		end
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

		qcDropDownInfo = UIDropDownMenu_CreateInfo()
		qcDropDownInfo.text = tostring(QCTRIPLEPADDING .. "Sort: Alphabetically")
		qcDropDownInfo.notCheckable = true
		qcDropDownInfo.colorCode = COLOUR_DRUID
		qcDropDownInfo.arg1 = "SORTAPLHA"
		function qcDropDownInfo.func(button, arg1)
			qcSettings["SORT"] = 2
			qcUpdateMenu(qcCurrentZone, qcMenuSlider:GetValue())
		end
		UIDropDownMenu_AddButton(qcDropDownInfo, level)

	elseif (level == 2) then

		--[[ Kalimdor ]]--
		if (UIDROPDOWNMENU_MENU_VALUE == 1) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 1) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Eastern Kingdoms ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 2) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 2) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Outland ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 3) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 3) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Northrend ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 4) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 4) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ The Maelstrom ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 5) then
			for qcZoneIndex, qcZone in ipairs(qcContinentZones) do
				if (qcZone[1] == 5) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcZone[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcZone[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcZone[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Classic Dungeon ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 10) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 10) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ The Burning Crusade Dungeon ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 11) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 11) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Wrath of the Lich King Dungeon ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 12) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 12) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Cataclysm Dungeon ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 13) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 13) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Raid ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 14) then
			for qcDungeonIndex, qcDungeon in ipairs(qcDungeons) do
				if (qcDungeon[1] == 14) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcDungeon[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcDungeon[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcDungeon[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Classes ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 20) then
			for qcClassIndex, qcClass in ipairs(qcClasses) do
				if (qcClass[1] == 20) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcClass[3]
					qcDropDownInfo.notCheckable = false
					qcDropDownInfo.colorCode = qcClass[4]
					if (qcCurrentZone == qcClass[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcClass[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Professions ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 30) then
			for qcProfessionIndex, qcProfession in ipairs(qcProfessions) do
				if (qcProfession[1] == 30) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcProfession[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcProfession[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcProfession[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ World Events ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 31) then
			for qcWorldEventIndex, qcWorldEvent in ipairs(qcWorldEvents) do
				if (qcWorldEvent[1] == 31) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcWorldEvent[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcWorldEvent[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcWorldEvent[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Battlegrounds ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 32) then
			for qcBattlegroundIndex, qcBattleground in ipairs(qcBattlegrounds) do
				if (qcBattleground[1] == 32) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcBattleground[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcBattleground[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcBattleground[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		--[[ Specials ]]--
		elseif (UIDROPDOWNMENU_MENU_VALUE == 33) then
			for qcSpecialIndex, qcSpecial in ipairs(qcSpecials) do
				if (qcSpecial[1] == 33) then
					qcDropDownInfo = UIDropDownMenu_CreateInfo()
					qcDropDownInfo.text = qcSpecial[3]
					qcDropDownInfo.notCheckable = false
					if (qcCurrentZone == qcSpecial[2]) then
						qcDropDownInfo.checked = true
					end
					qcDropDownInfo.arg1 = qcSpecial[2]
					function qcDropDownInfo.func(button, arg1)
						qcUpdateMenu(arg1, 1)
						CloseDropDownMenus()
					end
					UIDropDownMenu_AddButton(qcDropDownInfo, level)
				end
			end
		end

	end

end

function qcZoneDropdownButton_OnClick(self, button, down)

	local name = self:GetName()
	ToggleDropDownMenu(1, nil, qcZoneDropdownMenu, name, 0, 0)

end

function qcZoneDropdownOnLoad(self)

	UIDropDownMenu_Initialize(self, qcZoneDropdown_Initialize)

end

function qcFactionCheckButton_OnClick()

	if (_G["qcFactionCheckButton"]:GetChecked() == nil) then
		qcSettings["HIDEOPPFACTION"] = 0
	else
		qcSettings["HIDEOPPFACTION"] = 1
	end

	qcUpdateMenu(qcCurrentZone, 1)

end

function qcClassCheckButton_OnClick()

	if (_G["qcClassCheckButton"]:GetChecked() == nil) then
		qcSettings["HIDEOTHERCLASSRACE"] = 0
	else
		qcSettings["HIDEOTHERCLASSRACE"] = 1
	end

	qcUpdateMenu(qcCurrentZone, 1)

end

function qcCompletedCheckButton_OnClick()

	if (_G["qcCompletedCheckButton"]:GetChecked() == nil) then
		qcSettings["HIDECOMPLETED"] = 0
	else
		qcSettings["HIDECOMPLETED"] = 1
	end

	qcUpdateMenu(qcCurrentZone, 1)

end

local function qcGetPlayerClassFaction()

	qcPlayerLocalizedClass, qcPlayerEnglishClass = UnitClass("player")
	qcPlayerEnglishFaction, qcPlayerLocalizedFaction = UnitFactionGroup("player")
	qcPlayerLocalizedRace, qcPlayerEnglishRace	= UnitRace("player")

end

local function qcGetZoneIDFromName(zoneName)

	--TODO: Quest Categories or Continent Zones?

	for qcZoneIndex, qcZone in ipairs(qcQuestCategories) do
		if (qcZone[2] == zoneName) then -- This seems to be the right one --
			return qcZone[1]
		end
	end

	-- TODO: Map Specials here

	return nil

end

local function qcZoneChangedNewArea()

	local qcZoneID = qcGetZoneIDFromName(GetRealZoneText())

	if (qcZoneID ~= nil) then
		qcUpdateMenu(qcZoneID, 1)
	end

end

local function qcRecievedAddonMessage(...)

local qcPrefix
local qcMessage
local qcChannel
local qcSender
local qcCleanedMessage
local qcCleanedArgument

	qcPrefix, qcMessage, qcChannel, qcSender = ...
	if (qcPrefix == "QCOMPLETIST") then
		qcCleanedMessage = string.sub(qcMessage, 1, (string.find(qcMessage, QCADDON_DELIMITER, 1, true) - 1))
		qcCleanedArgument = string.sub(qcMessage, (string.find(qcMessage, QCADDON_DELIMITER, 1, true) + 1), #qcMessage)

		if (qcCleanedMessage == "VERSIONCASCADE") then
			if (tonumber(qcCleanedArgument) > QCADDON_VERSION) then
				print(QCADDON_CHAT_TITLE .. string.format(L["QC_NEWVERSIONDETECTED"], QCADDON_VERSION, qcSender, qcCleanedArgument))
			end
		elseif (qcCleanedMessage == "REMOTELOGGER") then
			local qcCleanedArg1
			local qcCleanedArg2
			local qcCleanedArg3
			local qcCleanedArg4
			local qcCleanedArg5
			local qcCleanedArg6
			local qcCleanedArg7
			local qcCleanedArg8
			print(qcCleanedArgument)
		else
			--TODO
			print(QCADDON_CHAT_TITLE .. "Unknown addon message recieved from " .. qcSender .. ". It is possible you are running an old version of this addon.")
		end
	end

end

function qcUpdateTooltip(button)

local qcQuestID = _G["qcMenuButton" .. button .. "_QuestTag"]:GetText()
local qcQuestProviderID = _G["qcMenuButton" .. button .. "_QuestProviderID"]:GetText()
local qcQuestProviderName = _G["qcMenuButton" .. button .. "_QuestProviderName"]:GetText()
local qcQuestProviderZone = _G["qcMenuButton" .. button .. "_QuestProviderZone"]:GetText()
local qcQuestProviderX = _G["qcMenuButton" .. button .. "_QuestProviderX"]:GetText()
local qcQuestProviderY = _G["qcMenuButton" .. button .. "_QuestProviderY"]:GetText()
local qcQuestProviderComments = _G["qcMenuButton" .. button .. "_QuestProviderComments"]:GetText()

if (qcQuestID ~= nil) then

	GameTooltip:SetOwner(frameQuestCompletist, "ANCHOR_BOTTOMRIGHT", -30, 500)
	GameTooltip:ClearLines()
	GameTooltip:SetHyperlink("quest:" .. qcQuestID)
	GameTooltip:AddLine(" ")
	GameTooltip:AddDoubleLine("Quest ID:", COLOUR_MAGE .. qcQuestID)

	if (qcQuestProviderID ~= nil) and (qcQuestProviderName ~= nil) then
		GameTooltip:AddDoubleLine("Quest Giver:", COLOUR_MAGE .. qcQuestProviderName .. " [" .. qcQuestProviderID .. "]")
	end

	if (qcQuestProviderZone ~= nil) and (qcQuestProviderX ~= nil) and (qcQuestProviderY ~= nil) then
		GameTooltip:AddLine(" ")
		GameTooltip:AddDoubleLine(COLOUR_ROGUE .. qcQuestProviderZone, COLOUR_ROGUE .. qcQuestProviderX .. ", " .. qcQuestProviderY)
	end

	if (qcQuestProviderComments ~= nil) then
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(COLOUR_HUNTER .. qcQuestProviderComments, nil, nil, nil, true)
	end

	GameTooltip:Show()

end
end

function RunSlashCmd(cmd)

local slash, rest = cmd:match("^(%S+)%s*(.-)$")

	for name, func in pairs(SlashCmdList) do
		local i, slashCmd = 1
		repeat
		slashCmd, i = _G["SLASH_"..name..i], i + 1
			if slashCmd == slash then
				return true, func(rest)
			end
		until not slashCmd
	end

local i = 1

	while _G["EMOTE" .. i .. "_TOKEN"] do
		local j, cn = 2, _G["EMOTE" .. i .. "_CMD1"]
		while cn do
			if cn == slash then
				return true, DoEmote(_G["EMOTE" .. i .. "_TOKEN"], rest)
			end
			j, cn = j+1, _G["EMOTE" .. i .. "_CMD" .. j]
		end
	i = i + 1
	end

end

function qcQuestClick(buttonnumber)

local qcQuestID = _G["qcMenuButton" .. buttonnumber .. "_QuestTag"]:GetText()
local qcQuestProviderID = _G["qcMenuButton" .. buttonnumber .. "_QuestProviderID"]:GetText()
local qcQuestProviderName = _G["qcMenuButton" .. buttonnumber .. "_QuestProviderName"]:GetText()
local qcQuestProviderZone = _G["qcMenuButton" .. buttonnumber .. "_QuestProviderZone"]:GetText()
local qcQuestProviderX = _G["qcMenuButton" .. buttonnumber .. "_QuestProviderX"]:GetText()
local qcQuestProviderY = _G["qcMenuButton" .. buttonnumber .. "_QuestProviderY"]:GetText()
local qcQuestProviderComments = _G["qcMenuButton" .. buttonnumber .. "_QuestProviderComments"]:GetText()

	if (IsLeftShiftKeyDown() == nil) then
		if IsAddOnLoaded('TomTom') then
			if (qcQuestProviderZone == nil) or (qcQuestProviderX == nil) or (qcQuestProviderY == nil) then
				return nil
			else
				if (qcQuestProviderName == nil) then
					qcQuestProviderName = "Quest Giver"
				end
				RunSlashCmd("/tway " .. qcQuestProviderZone .. " " .. qcQuestProviderX .. " " .. qcQuestProviderY .. " " .. qcQuestProviderName .. " in " .. qcQuestProviderZone)
				RunSlashCmd("/cway")
			end
		end
	else --[[ User wants to toggle if the quest is completed ]]--
		if (qcCompletedQuests[tonumber(qcQuestID)] == nil) then
			qcCompletedQuests[tonumber(qcQuestID)] = {}
			qcCompletedQuests[tonumber(qcQuestID)]["C"] = 1 --[[ Completed ]]--
		else
			if (qcCompletedQuests[tonumber(qcQuestID)]["C"] == nil) then
				qcCompletedQuests[tonumber(qcQuestID)]["C"] = 1 --[[ Completed ]]--
			else
				if (qcCompletedQuests[tonumber(qcQuestID)]["C"] == 1) then
					qcCompletedQuests[tonumber(qcQuestID)]["C"] = 0
				elseif (qcCompletedQuests[tonumber(qcQuestID)]["C"] == 0) then
					qcCompletedQuests[tonumber(qcQuestID)]["C"] = 1
				end
			end
		end
		qcUpdateMenu(nil, qcMenuSlider:GetValue())
	end

end

function qcCloseTooltip()

	GameTooltip:Hide()

end

function qcRemoteLogger(qcQuestID, qcQuestTitle, qcPlayerXPos, qcPlayerYPos, qcNPCGUID, qcNPCID, qcNPCName, qcQuestCategory)

local inGuild
local QUEST_DELIMITER = "|"
local qcQuestData = qcQuestID .. QUEST_DELIMITER .. qcQuestTitle .. QUEST_DELIMITER .. qcPlayerXPos .. QUEST_DELIMITER .. qcPlayerYPos .. QUEST_DELIMITER .. qcNPCGUID .. QUEST_DELIMITER .. qcNPCID .. QUEST_DELIMITER .. qcNPCName .. QUEST_DELIMITER .. qcQuestCategory

	inGuild = IsInGuild()
	if (inGuild == 1) then
		SendAddonMessage(QCADDON_MSG_PREFIX, "REMOTELOGGER:" .. qcQuestData, "GUILD")
	end

end

local function qcVersionCascade()

local inGuild

	--[[ Initiate version cascade ]]--
	inGuild = IsInGuild()
	if (inGuild == 1) then
		SendAddonMessage(QCADDON_MSG_PREFIX, "VERSIONCASCADE:" .. QCADDON_VERSION, "GUILD")
	end

end

local function qcCheckSettings()

	if (qcSettings == nil) then
		qcSettings = {}
	end

	if (qcSettings["SORT"] == nil) then --[[ 1:Level, 2:Alpha, 3:Quest Giver ]]--
		qcSettings["SORT"] = 1
	end
	if (qcSettings["HIDEOPPFACTION"] == nil) then --[[ 1:Yes, 2:No ]]--
		qcSettings["HIDEOPPFACTION"] = 1
	end
	if (qcSettings["HIDEOTHERCLASSRACE"] == nil) then --[[ 1:Yes, 2:No ]]--
		qcSettings["HIDEOTHERCLASSRACE"] = 1
	end
	if (qcSettings["HIDECOMPLETED"] == nil) then --[[ 1:Yes, 2:No ]]--
		qcSettings["HIDECOMPLETED"] = 0
	end

end

local function qcApplySettings()

	_G["qcFactionCheckButtonText"]:SetText("Hide Opposing Forces Quests")
	if (qcSettings["HIDEOPPFACTION"] == 0) then
		_G["qcFactionCheckButton"]:SetChecked(false)
	else
		_G["qcFactionCheckButton"]:SetChecked(true)
	end

	_G["qcClassCheckButtonText"]:SetText("Hide Quests for Other Classes (Not Yet Implimented)")
	if (qcSettings["HIDEOTHERCLASSRACE"] == 0) then
		_G["qcClassCheckButton"]:SetChecked(false)
	else
		_G["qcClassCheckButton"]:SetChecked(true)
	end

	_G["qcCompletedCheckButtonText"]:SetText("Hide Completed Quests")
	if (qcSettings["HIDECOMPLETED"] == 0) then
		_G["qcCompletedCheckButton"]:SetChecked(false)
	else
		_G["qcCompletedCheckButton"]:SetChecked(true)
	end

end

local function qcUpdateCompletedQuest(qcQuestID)

	if (qcCompletedQuests[qcQuestID] == nil) then --[[ The quest isn't already marked as completed ]]--
		qcCompletedQuests[qcQuestID] = {}
		qcCompletedQuests[qcQuestID]["C"] = 1
	else --[[ The quest is already marked as completed ]]--
		--[[ Possible future update code ]]--
	end

end

function qcOnUpdate(self, elapsed)

	
	--qcOnUpdateElapsed = qcOnUpdateElapsed + elapsed

	--if (qcOnUpdateElapsed > qcOnUpdateInterval) then
	--	if (GetQuestResetTime() > qcTimeToDailyReset) then --[[ If the reset time has increased then it must have reset ]]--
	--		qcUpdateMenu(nil, qcMenuSlider:GetValue())
	--	end
	--	qcOnUpdateElapsed = 0
	--end

end

local function qcEventHandler(self, event, ...)

	if (event == "QUEST_COMPLETE") then

		local qcQuestID = GetQuestID() --[[ Snatch the quest ID from the current quest ]]--
		if (qcQuestID ~= 0) then --[[ For whatever reason, we didn't detect the quest ID ]]--
			qcUpdateCompletedQuest(qcQuestID)
			qcUpdateMenu(nil, qcMenuSlider:GetValue())
		end

	elseif (event == "QUEST_DETAIL") then
		qcBuildDynamicDB(false)
	elseif (event == "QUEST_ACCEPTED") then
		qcBuildDynamicDB(true)
		qcUpdateMenu(nil, qcMenuSlider:GetValue())
	elseif (event == "UNIT_QUEST_LOG_CHANGED") then
		if (... == "player") then
			qcUpdateMenu(nil, qcMenuSlider:GetValue())
		end
	elseif (event == "QUEST_QUERY_COMPLETE") then
		qcQuestQueryCompleted()
		frameQuestCompletist:UnregisterEvent("QUEST_QUERY_COMPLETE")
	elseif (event == "PLAYER_ENTERING_WORLD") then
		qcGetPlayerClassFaction()
		qcZoneChangedNewArea()
		qcVersionCascade()
	elseif (event == "ZONE_CHANGED_NEW_AREA") then
		qcZoneChangedNewArea()
	elseif (event == "CHAT_MSG_ADDON") then
		qcRecievedAddonMessage(...)
	elseif (event == "ADDON_LOADED") then
		if (... == "QuestCompletist") then
			qcCheckSettings()
			qcApplySettings()
			qcLoggerUpdate()
		end
	end

end

function frameQuestCompletist_OnLoad(self)

	--[[ Correct the portrait icon ]]--
	SetPortraitToTexture("frameQuestCompletist_Portrait", "Interface\\ICONS\\TRADE_ARCHAEOLOGY_DRAENEI_TOME")
	_G["frameQuestCompletist_titleText"]:SetText("Quest Completist v" .. QCADDON_VERSION)

	--[[ Register the frame so it can be dragged ]]--
	self:RegisterForDrag("LeftButton")

	--[[ Register for events ]]--
	self:RegisterEvent("QUEST_COMPLETE")
	self:RegisterEvent("QUEST_DETAIL")
	self:RegisterEvent("QUEST_ACCEPTED")
	self:RegisterEvent("UNIT_QUEST_LOG_CHANGED")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("CHAT_MSG_ADDON")
	self:RegisterEvent("ADDON_LOADED")

	--[[ Send events to the event handler ]]--
	self:SetScript("OnEvent", qcEventHandler)

	qcTimeToDailyReset = GetQuestResetTime()

end